@@ -1,11 +1,11 @@
 package Net::IMAP::Simple;
-# $Id: Simple.pm,v 1.3 2004/06/29 12:00:06 cwest Exp $
+# $Id: Simple.pm,v 1.4 2005/04/25 12:00:00 cfaber Exp $
 use strict;
-use IO::Socket;
 use IO::File;
+use IO::Socket;
 
 use vars qw[$VERSION];
-$VERSION = '0.95';
+$VERSION = '0.96';
 
 =head1 NAME
 
@@ -13,24 +13,38 @@ Net::IMAP::Simple - Perl extension for simple IMAP account handling.
 
 =head1 SYNOPSIS
 
+    # Duh
     use Net::IMAP::Simple;
     use Email::Simple;
 
-    my $server = Net::IMAP::Simple->new( 'someserver' );
-    $server->login( 'someuser', 'somepassword' );
-    
-    my $nmessages = $server->select( 'somefolder' );
+    # Create the object
+    my $imap = Net::IMAP::Simple->new('imap.example.com') ||
+       die "Unable to connect to IMAP: $Net::IMAP::Simple::errstr\n";
 
-    foreach my $msg ( 1 .. $number_of_messages ) {
-        print "This message has been read before...\n"
-          if $server->seen( $msg );
+    # Log on
+    my $msgs = $imap->login('user','pass');
 
-        my $email = Email::Simple->new( join '', @{$server->get( $msg )} );
-        
-        print $email->header('Subject'), "\n";
+    if($msgs !~ /^\d+$/){
+        print STDERR "Login failed: " . $imap->errstr . "\n";
+        exit(64);
     }
 
-    $server->quit();
+    # Print the subject's of all the messages in the INBOX
+    my $nm = $imap->select('INBOX');
+
+    for(my $i = 1; $i < $nm; $i++){
+        if($imap->seen($i)){
+            print "*";
+        } else {
+            print " ";
+        }
+
+        my $es = Email::Simple->new(join '', @{ $imap->top($i) } );
+
+        printf("[%03d] %s\n", $i, $es->header('Subject'));
+    }
+
+    $imap->quit;
 
 =head1 DESCRIPTION
 
@@ -42,52 +56,113 @@ This module is a simple way to access IMAP accounts.
 
 =item new
 
-  my $imap = Net::IMAP::Simple->new( $server );
 
-This class method constructs a new C<Net::IMAP::Simple> object. It takes
-one required parameter, the server to connect to. The parameter may
-specify just the server, or both the server and port number. To specify
-an alternate port, seperate it from the server with a colon (C<:>),
-C<example.com:5143>.
+my $imap = Net::IMAP::Simple->new( $server [ :port ] ) 
 
-On success an object is returned. On failure, C<undef> is returned.
+ OR 
+
+my $imap = Net::IMAP::Simple->new( $server [, option_name => option_value ] );
+
+This class method constructs a new C<Net::IMAP::Simple> object. It takes one required parameter which is the server to connect to, and additional optional parameters.
+
+The server parameter may specify just the server, or both the server and port number. To specify an alternate port, seperate it from the server with a colon (C<:>), C<example.com:5143>.
+
+On success an object is returned. On failure, nothing is returned and an error message is set to $Net::IMAP::Simple.
+
+Options:
+
+ port        => some port other than 143
+ timeout     => connection timeout after (x) seconds
+ retry       => try and reconnect (x) times
+ retry_delay => wait (x) seconds before retrying
+ use_v6      => 1|0 -- Use IPv6 sockets rather than IPv4
+ bindaddr    => some local address to bind
 
 =cut
 
 sub new {
-    my ( $class, $server) = @_;
+    my ( $class, $server, %opts) = @_;
 
     my $self = bless {
         count => -1,
     } => $class;
-    
-    my $connect  = $server;
-       $connect .= ':' . $self->_port if index($connect, ':') == -1;
 
-    $self->{sock} = $self->_sock_from->new( $connect )
-        or return;
+    my ($srv, $prt) = split(/:/, $server, 2);
+    $prt ||= ($opts{port} ? $opts{port} : $self->_port);
+
+    $self->{server} = $srv;
+    $self->{port} = $prt;
+    $self->{timeout} ||= 90;
+    $self->{use_v6} = ($opts{use_v6} ? 1 : 0);
+    $self->{retry} ||= 1;
+    $self->{retry_delay} ||= 5;
+    $self->{bindaddr} = $opts{bindaddr};
+
+    my $c;
+    for(my $i = 0; $i < $self->{retry}; $i++){
+	if($self->{sock} = $self->_connect){
+		$c = 1;
+		last;
+	}
+    }
+
+    if(!$c){
+	$@ =~ s/IO::Socket::INET6?: //g;
+	$Net::IMAP::Simple::errstr = "connection failed $@";
+	return;
+    }
+
+
     $self->_sock->getline();
 
     return $self;
 }
-sub _port         { 143                }
-sub _sock_from    { 'IO::Socket::INET' }
-sub _sock         { $_[0]->{sock}      }
-sub _count        { $_[0]->{count}     }
-sub _last         { $_[0]->{last}      }
+
+sub _connect {
+ my ($self) = @_;
+ my $sock;
+ if($self->{use_v6}){
+	require 'IO::Socket::INET6';
+	import IO::Socket::INET6;
+
+	$sock = $self->_sock_from_v6->new(
+		PeerAddr => $self->{server},
+		PeerPort => $self->{port},
+		Timeout  => $self->{timeout},
+		Proto    => 'tcp6',
+		($self->{bindaddr} ? { LocalAddr => $self->{bindaddr} } : '')
+	);
+ } else {
+	$sock = $self->_sock_from->new(
+		PeerAddr => $self->{server},
+		PeerPort => $self->{port},
+		Timeout  => $self->{timeout},
+		Proto    => 'tcp',
+		($self->{bindaddr} ? { LocalAddr => $self->{bindaddr} } : '')
+	);
+ }
+
+ return $sock;
+}
+
+sub _port         { 143                 }
+sub _sock_from    { 'IO::Socket::INET'  }
+sub _sock_from_v6 { 'IO::Socket::INET6' }
+sub _sock         { $_[0]->{sock}       }
+sub _count        { $_[0]->{count}      }
+sub _last         { $_[0]->{last}       }
 
 =pod
 
 =item login
 
-  my $inbox_msgs = $imap->login($user => $passwd);
+  my $inbox_msgs = $imap->login($user, $passwd);
 
 This method takes two required parameters, a username and password. This pair
 is authenticated against the server. If authentication is successful the
 user's INBOX is selected.
 
-On success, the number of messages in the INBOX is returned. C<undef> is returned
-on failure.
+On success, an integer which represents the number of messages in the INBOX is returned. Nothing is returned on failure and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -107,8 +182,7 @@ sub login {
 
     my $num_messages = $imap->select($folder);
 
-Selects a folder named in the single required parameter. The number of messages
-in that folder is returned on success. On failure, C<undef> is returned.
+Selects a folder named in the single required parameter. The number of messages in that folder is returned on success. On failure, nothing is returned  and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -132,8 +206,8 @@ sub select {
 
 This method accepts a message number as its required parameter. That message
 will be retrieved from the currently selected folder. On success this method
-returns a list reference containing the lines of the header. C<undef> is
-returned on failure.
+returns a list reference containing the lines of the header. Nothing is
+returned on failure and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -156,7 +230,7 @@ sub top {
 
 A message number is the only required parameter for this method. The message's
 C<\Seen> flag will be examined and if the message has been seen a true
-value is returned. All other failures return a false value.
+value is returned. All other failures return a false value and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -181,7 +255,7 @@ sub seen {
 This method returns size information for a message, as indicated in the
 single optional parameter, or all messages in a mailbox. When querying a
 single message a scalar value is returned. When listing the entire
-mailbox a hash is returned. On failure, C<undef> is returned.
+mailbox a hash is returned. On failure, nothing is returned and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -209,7 +283,7 @@ sub list {
   print for @{$message};
 
 This method fetches a message and returns its lines in a list reference.
-On failure, C<undef> is returned.
+On failure, nothing is returned and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -233,7 +307,7 @@ sub get {
   print <$file>;
 
 On success this method returns a file handle pointing to the message
-identified by the required parameter. On failure, C<undef> is returned.
+identified by the required parameter. On failure, nothing is returned and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -259,7 +333,7 @@ sub getfh {
   $item->quit;
 
 This method logs out of the IMAP server, expunges the selected mailbox,
-and closes the connection.
+and closes the connection. No error message will ever be returned from this method.
 
 =cut
 
@@ -278,8 +352,8 @@ sub quit {
   my $message_number = $imap->last;
 
 This method retuns the message number of the last message in the selected
-mailbox, since the last time the mailbox was selected. On failure, C<undef>
-is returned.
+mailbox, since the last time the mailbox was selected. On failure, nothing
+is returned and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -292,7 +366,7 @@ sub last { shift->_last }
   print "Gone!" if $imap->delete( $message_number );
 
 This method deletes a message from the selected mailbox. On success it
-returns true. False on failure.
+returns true. False on failure and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -334,6 +408,8 @@ recurses from the IMAP root to get all mailboxes. The first optional
 argument is a mailbox path and the second is the path reference. RFC 3501
 has more information.
 
+On failure nothing is returned and the errstr() error handler is set with the error message.
+
 =cut
 
 sub mailboxes {
@@ -365,7 +441,7 @@ sub mailboxes {
   print "Created" if $imap->create_mailbox( "/Mail/lists/perl/advocacy" );
 
 This method creates the mailbox named in the required argument. Returns true
-on success, false on failure.
+on success, false on failure and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -387,7 +463,7 @@ sub create_mailbox {
   print "Expunged" if $imap->expunge_mailbox( "/Mail/lists/perl/advocacy" );
 
 This method removes all mail marked as deleted in the mailbox named in
-the required argument. Returns true on success, false on failure.
+the required argument. Returns true on success, false on failure and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -409,7 +485,7 @@ sub expunge_mailbox {
   print "Deleted" if $imap->delete_mailbox( "/Mail/lists/perl/advocacy" );
 
 This method deletes the mailbox named in the required argument. Returns true
-on success, false on failure.
+on success, false on failure and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -432,7 +508,7 @@ sub delete_mailbox {
 
 This method renames the mailbox in the first required argument to the
 mailbox named in the second required argument. Returns true on success,
-false on failure.
+false on failure and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -456,7 +532,7 @@ sub rename_mailbox {
 
 This method copies the message number in the currently seleted mailbox to
 the fold specified in the second argument. Both arguments are required. On
-success this method returns true. Returns false on failure.
+success this method returns true. Returns false on failure and the errstr() error handler is set with the error message.
 
 =cut
 
@@ -471,18 +547,31 @@ sub copy {
     );
 }
 
+=item errstr
+
+Return the last error string captured for the last operation which failed.
+
+=cut
+
+sub errstr {
+ return $_[0]->{_errstr};
+}
+
 sub _nextid       { ++$_[0]->{count}   }
+
 sub _escape {
     $_[0] =~ s/\\/\\\\/g;
     $_[0] =~ s/\"/\\\"/g;
     $_[0] = "\"$_[0]\"";
 }
+
 sub _unescape {
     $_[0] =~ s/^"//g;
     $_[0] =~ s/"$//g;
     $_[0] =~ s/\\\"/\"/g;
     $_[0] =~ s/\\\\/\\/g;
 }
+
 sub _send_cmd {
     my ( $self, $name, $value ) = @_;
     my $sock = $self->_sock;
@@ -491,13 +580,22 @@ sub _send_cmd {
     { local $\; print $sock $cmd; }
     return ($sock => $id);
 }
+
 sub _cmd_ok {
     my ($self, $res) = @_;
     my $id = $self->_count;
     return 1 if $res =~ /^$id\s+OK/i;
-    return 0 if $res =~ /^$id\s+(?:NO|BAD)/i;
-    return undef;
+    if($res =~ /^$id\s+(?:NO|BAD)(?:\s+(.+))?/i){
+	$self->_seterrstr($1 || 'unknown error');
+	return 0;
+    } else {
+	$self->_seterrstr("unknown return string: $res");
+	return;
+    }
+
+    return;
 }
+
 sub _process_cmd {
     my ($self, %args) = @_;
     my ($sock, $id) = $self->_send_cmd(@{$args{cmd}});
@@ -515,6 +613,13 @@ sub _process_cmd {
     }
 }
 
+sub _seterrstr {
+ my ($self, $err) = @_;
+ $self->{_errstr} = $err;
+ return;
+}
+
+
 =pod
 
 =back
@@ -527,6 +632,8 @@ __END__
 
 =head1 AUTHOR
 
+Colin Faber, <F<cfaber@fpsn.net>>.
+
 Casey West, <F<casey@geeknst.com>>.
 
 Joao Fonseca, <F<joao_g_fonseca@yahoo.com>>.
@@ -534,10 +641,12 @@ Joao Fonseca, <F<joao_g_fonseca@yahoo.com>>.
 =head1 SEE ALSO
 
 L<Net::IMAP>,
-L<perl>.
+L<perl>,
+L<Changes>
 
 =head1 COPYRIGHT
 
+Copyright (c) 2005 Colin Faber.
 Copyright (c) 2004 Casey West.
 Copyright (c) 1999 Joao Fonseca.
 
@@ -546,4 +655,3 @@ redistribute it and/or modify it under the same terms as Perl
 itself.
 
 =cut
-
